using System;
using Jypeli;

namespace avaruuspeli;

/// @author ellatarkka
/// @version 03.04.2025
/// <summary>
/// Tässä pelissä avauusraketti koittaa kerätä avaruudesta tähtiä ja planeettoja ja vältellä kiviä.
/// Näistä pelaaja saa pisteitä. Jos raketti osuu kiveen avaruudessa se tuhoutuu.
/// Tämän jälkeen pelaaja näkee omat pisteet ja voi aloittaa pelin uudelleen.
/// https://tim.jyu.fi/view/kurssit/tie/ohj1/v/2025/kevat/demot/demo7?answerNumber=2&b=QwUjtPvVmkkl&size=1&task=d3teht6a&user=tarkkael
/// Tässä on linkki demo 7 tehtävään 1, jossa käytän funktiota
/// </summary>
public class Avaruuspeli : PhysicsGame
{
    private Image _olionKuva1 = LoadImage("Kuvat/tähti"); 
    private Image _olionKuva2 = LoadImage("Kuvat/kivi"); 
    private Image _olionKuva3 = LoadImage("Kuvat/raketti"); 
    private Image _olionKuva4 = LoadImage("Kuvat/planeetta2");
    
    // vakiodaan objektien koko
    private const double KOKO = 100.0;
    
    private PhysicsObject tahti;
    private PhysicsObject kivi;
    private PhysicsObject planeetta;
    private PhysicsObject rakettI;
    
    Vector nopeusVasen = new Vector(-200, 0);
    Vector nopeusOikea = new Vector(200, 0);
    
    private IntMeter pisteet;
    private Timer spawnTimer;
        
    /// <summary>
    /// Pääohjelma.
    /// </summary>
    [STAThread]
    public override void Begin()
    {
        {
            LuoKentta();
            
            AloitaPeli();
            AsetaOhjaimet();
            LisaaLakuri();
            AloitaSpawn();
            
            
            AddCollisionHandler(rakettI, tahti, RakettiOsuiTahtiin);
            AddCollisionHandler(rakettI, planeetta, RakettiOsuiPlaneettaan);
            AddCollisionHandler(rakettI, kivi, RakettiOsuiKiveen);
        }
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
    
    
    /// <summary>
    /// Luodaan pelin kentta ja sinne kuuluvat oliot ja ominaisuudet.
    /// Tämä aliohjelma alustaa pelikentän ja raketin, antamalla niille tarvittavat ominasuudet toimiakseen. 
    /// </summary>
    /// <returns></returns>
    private void LuoKentta()
    {
        // Tehdään avaruuteen tähtiä
        Level.Background.CreateStars();
            
        // luodaan raketti, annetaan sille koko, paikka ja piirretty ulkomuoto.
        // Lisäksi annetaan sille käsky pysyä paikoillaan, muuten raketti pyörii ja valuu alaspäin, kun se osuu muihin oloiohin
        rakettI = new PhysicsObject(KOKO, KOKO);
        rakettI.X = 0;
        rakettI.Y = Level.Bottom + 40;
        rakettI.CanRotate = false;
        rakettI.IgnoresCollisionResponse = true;
        Add(rakettI);
        rakettI.Image = _olionKuva3;
        
        // kutsutaan objekteja luovia aliohjelmaa 
        planeetta = LuoObjekti(100, 200, _olionKuva4, "planeetta");
        tahti = LuoObjekti(-200, 300, _olionKuva1, "tahti");
        kivi = LuoObjekti(200, 30, _olionKuva2, "kivi");
        
        // Tehdään talukko missä käydään läpi reunat 
        PhysicsObject[] reunat =
        {
            Level.CreateLeftBorder(),
            Level.CreateRightBorder()
        };
        foreach (PhysicsObject reuna in reunat)
        {
            reuna.Restitution = 1.0;
            reuna.KineticFriction = 0.0;
            reuna.IsVisible = false;
        }

        // Tausta avaruuden väriseksi :)
        Level.Background.Color = Color.Black;
    }

    
    /// <summary>
    /// Aliohjelma luo objektin alla olevilla parametreillä
    /// Lisää objektin kentälle add metodilla
    /// </summary>
    /// <param name="x">objektin X kordinaatti</param>
    /// <param name="y">objektin Y kordinaatti</param>
    /// <param name="kuva">kuvan nimi, joka viittaa olioon</param>
    /// <param name="tag">oliolle määritetty tagi</param>
    /// <returns>palauttaa luodun olion</returns>
    private PhysicsObject LuoObjekti(double x, double y, Image kuva, string tag)
    {
        PhysicsObject olio = new PhysicsObject(KOKO, KOKO);
        olio.X = x; 
        olio.Y = y;
        olio.Tag = tag; 
        olio.Image = kuva; 
        Add(olio);
        return olio;
    }

    
    /// <summary>
    /// Aloitetaan peli laittamalla pelin objektit liikkumaan 
    /// </summary>
    private void AloitaPeli()
    { 
        Vector liikkuu = new Vector(0, -300);
        tahti.Move(liikkuu);
        kivi.Move(liikkuu);  
        planeetta.Move(liikkuu); 
    }
        
        
    /// <summary>
    /// Asetetaan näppäimistölle ohjaimet millä rakettia voi liikuttaa, ja kutsutaan aliohjelmaa joka asettaa nopeuden.
    /// Annetaan myös käsky pysyä paikoillaan kun ohjaimia on painettu, muuten raketti jatkaa matkaa ohjattuun suuntaan, eikä ole helposti ohjattavissa.
    /// </summary>
    private void AsetaOhjaimet()
    { 
        Keyboard.Listen(Key.Left, ButtonState.Down, AsetaNopeus, "raektti liikkuu vasemmalle", rakettI, nopeusVasen); 
        Keyboard.Listen(Key.Right, ButtonState.Down, AsetaNopeus, "raektti liikkuu oikealle", rakettI, nopeusOikea);
        Keyboard.Listen(Key.Left, ButtonState.Released, AsetaNopeus, "raketti pysähtyy", rakettI, Vector.Zero);
        Keyboard.Listen(Key.Right, ButtonState.Released, AsetaNopeus, "raketti pysähtyy", rakettI, Vector.Zero); 
    }
        
        
    /// <summary>
    /// Tehdään aliohjelma mikä asettaa raketin liikkeelle nopeuden silmukan avulla.
    /// Asetetaan toiminto joka lopettaa raketin liikkeen, jos se osuu reunoihin. 
    /// </summary>
    /// <param name="olio"></param>
    /// <param name="nopeus"></param>
    private void AsetaNopeus(PhysicsObject olio, Vector nopeus)
    {
        olio.Velocity = nopeus;
        if ((nopeus.X < 0) && (olio.Left < Level.Left))
        {
            olio.Velocity = Vector.Zero;
            return;
        }

        if ((nopeus.X > 0) && (olio.Right > Level.Right))
        {
            olio.Velocity = Vector.Zero;
            return;
        }
        olio.Velocity = nopeus;
    }
    
        
    /// <summary>
    /// Törmäyskäsittelijät antavat toimintoja kun raketti osuu putoaviin objekteihin.
    /// Törmäyksessä saa pisteitä tai kiveen osuttaessa peli loppuu ja tulee lopetusnäyttö.
    /// Osuessa tähteen pelaaja saa 1 pisteen ja tähti tuhoutuu.
    /// </summary>
    /// <param name="raketti"></param>
    /// <param name="osuttu"></param>
    private void RakettiOsuiTahtiin(PhysicsObject raketti, PhysicsObject osuttu)
    {
        pisteet.Value += 1;
        osuttu.Destroy();
    }

        
    /// <summary>
    /// osuessa planeettaan saa 2 pistettä ja planeetta tuhoutuu.
    /// </summary>
    /// <param name="raketti"></param>
    /// <param name="osuttu"></param>
    private void RakettiOsuiPlaneettaan(PhysicsObject raketti, PhysicsObject osuttu)
    {
        pisteet.Value += 2;
        osuttu.Destroy();
    }
        
        
    /// <summary>
    /// Törmätessä kiveen peli loppuu ja tulee lopetusnäyttö.
    /// </summary>
    /// <param name="raketti"></param>
    /// <param name="osuttu"></param>
   private void RakettiOsuiKiveen(PhysicsObject raketti, PhysicsObject osuttu)
    {
        ClearAll();
        spawnTimer.Stop();
        LopetusTekstit(); 
    }   
        
    
    /// <summary>
    /// Tehdään laskuri pelin näytölle, joka seuraa pelaajan pisteitä.
    /// Annetaan laskurille ominaisuuksia kuten paikka ja väri.
    /// </summary>
    /// <param name="x"></param>
    /// <param name="y"></param>
    /// <returns></returns>
    IntMeter LuoPistelaskuri(double x, double y)
    {
        IntMeter laskuri = new IntMeter(0);
        
        Label naytto = new Label();
        naytto.BindTo(laskuri);
        naytto.X = x;
        naytto.Y = y;
        naytto.TextColor = Color.Black;
        naytto.BorderColor = Color.Aqua;
        naytto.Color = Color.Aqua;
        Add(naytto);

        return laskuri;
    }
        
        
    /// <summary>
    /// Lisätään pistelaskuri peliin aliohjelman avulla.
    /// </summary>
    private void LisaaLakuri()
    {
        pisteet = LuoPistelaskuri(Screen.Right - 100.0, Screen.Top - 100.0);
    }


    /// <summary>
    /// Tämä aliohjelma luo tippuvia oloita sattumanvaraisesti pelikentälle 
    /// Tässä aliohjelmassa kutsumme tehtyjä objekteja aliohjelmien avulla, kuten tähteä, kiveä ja planeettaa
    /// </summary>
    private void SpawnaaObjekti()
    {
        int tyyppi = RandomGen.NextInt(0, 3);
        PhysicsObject obj;
        switch (tyyppi)
        {
            case 0:
                obj = LuoObjekti(RandomGen.NextDouble(Level.Left + 50, Level.Right - 50), Level.Top + KOKO, _olionKuva1,
                    "tahti");
                break;
            case 1:
                obj = LuoObjekti(RandomGen.NextDouble(Level.Left + 50, Level.Right - 50), Level.Top + KOKO, _olionKuva4,
                    "planeetta");
                break;
            case 2:
                obj = LuoObjekti(RandomGen.NextDouble(Level.Left + 50, Level.Right - 50), Level.Top + KOKO, _olionKuva2,
                    "kivi");
                break;
            default:
                return;
        }
        //objektin nopeus 
    obj.Velocity = new Vector(0, -300);

    //Annetaan ohjeet oloille, kun ne törmäävät rakettiin 
    if ((string)obj.Tag == "tahti")
        {
            AddCollisionHandler(rakettI, obj, RakettiOsuiTahtiin);
        }
        else if ((string)obj.Tag == "planeetta")
        {
            AddCollisionHandler(rakettI, obj, RakettiOsuiPlaneettaan);
        }
        else if ((string)obj.Tag == "kivi")
        {
            AddCollisionHandler(rakettI, obj, RakettiOsuiKiveen);
                
        }
    }
        
        
    /// <summary>
    /// Aliohjelma luo olion joka kutsuu spawnaaObjekti aliohjelmaa, joka lisää uusia objekteja satunnaisessa järjestyksessä.
    /// </summary>
    private void AloitaSpawn()
    {
        spawnTimer = new Timer();
        spawnTimer.Interval = 0.5;
        spawnTimer.Timeout += delegate { SpawnaaObjekti(); };
        spawnTimer.Start();
    }

        
        
    /// <summary>
    /// Tehdään aliohjelma siihen, kun peli loppuu. 
    /// Tämä aliohjelma näyttää pelaajalle saamansa pisteet ja tekstin jonka avulla pelaaja voi aloittaa pelin uudelleen painamalla enter.
    /// </summary>
    private void LopetusTekstit() 
    { 
        Label otsikko = new Label();
        otsikko.Text = "Peli ohi! Sait " + pisteet.Value + " pistettä."; 
        otsikko.X = Screen.Center.X; 
        otsikko.Y = Screen.Center.Y + 30;
        otsikko.Font.Size = 24;
        otsikko.TextColor = Color.White;
        Add(otsikko);
        
        Label otsikko2 = new Label();
        otsikko2.Text = "Aloita uusi peli painamalla enter";
        otsikko2.X = 0;
        otsikko2.Y = -70;
        otsikko2.Font.Size = 24;
        otsikko2.TextColor = Color.White;
        Add(otsikko2); 
        Keyboard.Listen(Key.Enter, ButtonState.Pressed, AloitaAlusta, "Aloita uusi peli");
    }
        
        
    /// <summary>
    /// Tehdään aliohjelma jota voidaan kutsua kun halutaan aloittaa peli alusta. 
    /// Esimerkiksi kun raketti on törmännyt kiveen haluamme aloittaa plein alusta painamalla enteriä.
    /// </summary>
    private void AloitaAlusta()
    { 
        ClearAll();
        Begin();
    }
}
